  ; dd 25.02.2005
  ;
  ; Erzeugt mit einer Nummernkreis-Definition eine neue Nummer und speichert diese ggf. sofort
  ; in denzugehrigen Datensatz. Mit dieser Routine soll verhindert werden, dass doppelte Nummern
  ; im System entstehen knnen.
  ;
  ; Der Nummernkreis wird whrend des Vorgangs gelockt. Sollte die Routine aufgerufen werden und
  ; der Nummernkreis bereits gelockt sein, werden solange Versuche unternommen, bis abgebrochen
  ; wird.
  ;
  ; Sollte der Nummernkreis gelockt werden und das Erzeugen einer neuen Nummer fehlschlagen, wir der
  ; Nummernkreis auf jeden Fall wieder freigegeben.
  ;
  ; Die Routine wird bentigt, da es viele verschieden Zhlweisen zu untersttzen gilt und auerdem
  ; Tabellen in der Datenbank existieren knnen, in denen in verschiedenen Feldern verschiedene Nummernkreise
  ; gezhlt werden mssen. Deswegen mu nicht in jedem Datensatz in jedem Zhlfeld ein Wert enthalten sein.
  ; Daher kann man diese Felder nicht mit einem UNIQUE-Datenbank-Schlssel sichern.
  ;
  ;exportiert PROCEDURE NewNum    VN NewNum.NumberID,   --> Welcher Nummernkreis
  ;                               VS NewNum.VarName,    --> Name des Maskenfeldes oder der Variable, die das Ergebnis enthalten soll.
  ;                               VF NewNum.RecID,      --> RecordID des Datensatzes, der die neue Nummern erhlt (kann leer sein).
  ;                                                         Wenn im "schnellen Modus" gezhlt wird, zeigt ein Wert > 0 an, dass
  ;                                                         das Ergebnis in den Startwert der Nummernkreisdefinition zurck
  ;                                                         geschrieben werden soll. Nur wenn erfolgreich eine neue Nummer erzeugt
  ;                                                         wurde und wenn dieser Wert NICHT > 0 ist, wird der Nummernkreis NICHT
  ;                                                         wieder entlockt bzw. freigegeben. Es muss dann explizid die Routine
  ;                                                         FreeNum aufgerufen werden.
  ;                                                         Achtung: Sollten in einem Script mehr als ein Nummernkreis gezhlt werden,
  ;                                                                  so ist darauf zu achten, dass der zuerst gelockte Nummernkeis
  ;                                                                  zuletzt entlockt bzw. freigegeben wird!
  ;                              ~VB NewNum.MainRes     --> Ergebnis.
  ;                                                         Wurde eine neue Nummer erzeugt (TRUE/FALSE) ?
  ;                                                         Bei fehlender Nummernkreisdefinition ist das Ergebnis TRUE
  ;
  ;
  ; Freigabe bzw. Entlocken eines Nummernkreises.
  ;
  ;            PROCEDURE FreeNum  VN FreeNum.NumberID,  --> Welcher Nummernkreis?
  ;                               VS FreeNum.VarName,   --> Name des Maskenfeldes oder der Variable, die ggf initialisiert werden soll.
  ;                              ~VB FreeNum.MainRes    --> Ergebnis. Wurde eine neue Nummernkreis freigegeben? (TRUE/FALSE)
  ;
  ;
  ;
  ; Gibt in [BlkNumber.Result] zurck, ob die Eingabefelder des Nummernkreises mit der ID [BlkNumber.NumberID] geblockt werden sollen oder nicht.
  ;
  ;            PROCEDURE BlkNum   VN BlkNum.NumberID,   --> Welcher Nummernkreis?
  ;                              ~VB BlkNum.Result      --> Soll der Numemrnkreis geblockt werden? Ja oder nein
  ;
  ;
  ;
  ; Ist eine Nummer in einem gegebenen Feld in einer Tabelle schon vergeben. Diese Routine bezieht sich NICHT auf die Nummernkrteise
  ; denn sie soll auch dann noch funktionieren, wenn eien User einen Nummernkreis gelscht hat.
  ;
  ;            PROCEDURE ChkNum   VS ChkNum.VarName,    --> Name des Maskenfeldes oder der Variable, das eine Nummer enthlt
  ;                               VF ChkNum.RecID,      --> RecordID des Datensatzes, der die Nummern erhlt (kann leer sein)
  ;                               VS ChkNum.TName       --> Tabelle, die geprft werden soll
  ;                               VS ChkNum.FName       --> Feld in der Tabelle, die geprft werden soll
  ;                               VS ChkNum.WClause     --> Bedingung, die bercksichtigt werden soll, das SQL-Woert "AND" muss enthalten sein!
  ;                              ~VB ChkNum.Result      --> Ist die Nummern schon (in einem anderen Datensatz) vergeben
  ;
  ;


  PROCEDURE NewNum VN NewNum.NumberID, VS NewNum.VarName, VF NewNum.RecID, ~VB NewNum.MainRes
    CALC [NewNum.MainRes] AS FALSE

    LOCAL_BOOL  NewNum.BlockField,      FALSE
    LOCAL_BOOL  NewNum.Result,          FALSE
    LOCAL_BOOL  NewNum.FlagState,       FALSE
    LOCAL_BOOL  NewNum.FastMode,        FALSE

    IDENTIFIER_EXIST 'NewNum.ActLocState',[NewNum.Result]
    IF [NewNum.Result]=FALSE
      VAR_BOOL  NewNum.ActLocState
    END_IF
    CALC [NewNum.ActLocState] AS [RECORD_LOCKING_IS_ACTIVE]

    LOCAL_QUERY NewNum.AQuery

    LOCAL_STR   NewNum.ANumber,         ''
    LOCAL_STR   NewNum.MaskName,        ''
    LOCAL_STR   NewNum.MaskField,       ''
    LOCAL_STR   NewNum.DataField,       ''
    LOCAL_STR   NewNum.DataTable,       ''
    LOCAL_STR   NewNum.WhereClause,     ''
    LOCAL_STR   NewNum.SubOrderBy,      ''
    LOCAL_STR   NewNum.DefFix,          ''
    LOCAL_STR   NewNum.DefFld,          ''
    LOCAL_STR   NewNum.DefVar,          ''
    LOCAL_STR   NewNum.NumRead,         ''
    LOCAL_STR   NewNum.TmpStr,          ''
    LOCAL_STR   NewNum.SwpStr,          ''
    LOCAL_STR   NewNum.StartValue,      ''
    LOCAL_STR   NewNum.VariabelStart,   ''
    LOCAL_STR   NewNum.StartFld,        ''
    LOCAL_STR   NewNum.StrSign,         ''

    LOCAL_FLOAT NewNum.NumDefRecID,     0.0

    LOCAL_NUM   NewNum.YNRes,           0
    LOCAL_NUM   NewNum.TmpNum,          0
    LOCAL_NUM   NewNum.LenFix,          0
    LOCAL_NUM   NewNum.LenVar,          0
    LOCAL_NUM   NewNum.LenStart,        0
    LOCAL_NUM   NewNum.LenRest,         0
    LOCAL_NUM   NewNum.NumValue,        0
    LOCAL_NUM   NewNum.Chars,           0
    LOCAL_NUM   NewNum.StepSize,        0

    CALC [NewNum.StrSign] AS CHAR(#39)

    ;******************************************************************************************
    ; Zuerst die Definition lesen, wenn sie nicht da ist bzw. ein Fehler stattfindet, raus hier
    ;******************************************************************************************
    QUERY [NewNum.AQuery]
        SELECT Stellen, DatenFeld, Datenbank, FestStart, Bezeichnung,
               VariabelStart, Schrittweite, StartWert,
               Bedingung, KreisID, SchnellModus, EingabeBlocken,
               RecordID, RecFlag, EventFlag
          FROM nummern
         WHERE KreisID = [NewNum.NumberID]
      ORDER BY KreisID
    END_QUERY

    EXECUTE [NewNum.AQuery],[NewNum.Result]
    IF [NewNum.Result]=FALSE
      MESSAGE 'Achtung schwerwiegender Fehler (1)','Die Nummernkreis-Definitionen knnen nicht abgefragt werden',[MESSAGE_ERROR]
      EXIT_PROC
    END_IF

    GET_DATA [NewNum.AQuery],[NewNum.Result]
    IF [NewNum.Result]=FALSE
      ; keine Fehlermeldung, weil ja auch Nummernkreise gelscht werden drfen
      CALC [NewNum.MainRes] AS TRUE
      EXIT_PROC
    END_IF
    CALC [NewNum.NumDefRecID] AS [NewNum.AQuery!RecordID]
    CALC [NewNum.BlockField]  AS [NewNum.AQuery!EingabeBlocken]

    IF [NewNum.AQuery!EventFlag]<>0
      MESSAGE 'Achtung schwerwiegender Fehler (2)','Der Nummernkreis ist defekt!',[MESSAGE_ERROR]
      EXIT_PROC
    END_IF


    ;******************************************************************************************
    ; Versuche den Nummernkreis zu loggen!
    ;******************************************************************************************
    CALC [WAIT_BREAK]    AS FALSE
    CALC [NewNum.SwpStr] AS CON('Ermitteln einer neuen Nummer fr den Nummernkreis "',[NewNum.AQuery!Bezeichnung],'"')
    WAIT_INFO 'Bitte warten...',[NewNum.SwpStr], TRUE

    IF [RECORD_LOCKING_IS_ACTIVE]=FALSE
      CALC [RECORD_LOCKING_IS_ACTIVE] AS TRUE
    END_IF

    CALC [NewNum.Result] AS FALSE
    WHILE [NewNum.Result]=FALSE
      IF [WAIT_BREAK]
        END_WAIT
        CALC [NewNum.YNRes]  AS [ID_NO]
        CALC [NewNum.TmpStr] AS CON('Soll das Ermitteln einer neuen Nummer fr den Nummernkreis "',[NewNum.AQuery!Bezeichnung],'" abgebrochen werden?')
        GET_YES_NO 'Achtung! Sind Sie sicher?!', [NewNum.TmpStr],[YNRes]
        IF [YNRes]=[ID_YES]
          CALC [RECORD_LOCKING_IS_ACTIVE] AS [NewNum.ActLocState]
          EXIT_PROC
        END_IF

        CALC [WAIT_BREAK]    AS FALSE
        CALC [NewNum.SwpStr] AS CON('Ermitteln einer neuen ',[NewNum.AQuery!Bezeichnung])
        WAIT_INFO 'Bitte warten...',[NewNum.SwpStr], TRUE
      END_IF

      RECORD_LOCK 'nummern',[NewNum.NumDefRecID],TRUE,[NewNum.Result],FALSE
      IF [NewNum.Result]=FALSE
        DELAY 500
      END_IF
    END_WHILE

    ;******************************************************************************************
    ; Wurde eine Masken-Variable an die Routine bergeben?
    ;******************************************************************************************
    CALC [NewNum.TmpNum]      AS POS('!',[NewNum.VarName])
    IF [NewNum.TmpNum]>0
      CALC [NewNum.MaskName]  AS PART([NewNum.VarName],1,'!')
      CALC [NewNum.MaskField] AS PART([NewNum.VarName],2,'!')
    END_IF

    ;******************************************************************************************
    ; Gibt es eine Bedingung? Sollen Bedinungsfelder in die ORDER BY Anweisung?
    ;******************************************************************************************
    IF [NewNum.AQuery!Bedingung]<>''
      CALC [NewNum.WhereClause] AS CON('WHERE ',[NewNum.AQuery!Bedingung])

      ; Alle >|< durch >"< ersetzten, so knnen auch Anfhrungszeichen mit in die Bedingung eingegeben und gespeichert werden
      CALC [NewNum.WhereClause] AS REPLACE([NewNum.WhereClause],'|', '"')
      CALC [NewNum.WhereClause] AS REPLACE([NewNum.WhereClause],[NewNum.StrSign], '"')

      ; Ein Ausrufungszeichen steht immer vor einem Bedinungs-Feld-Namen
      ; das in die Order-By-Clause des Such-Querys aufgenommen werden soll
      CALC [NewNum.TmpNum] AS POS('!',[NewNum.WhereClause])
      WHILE [NewNum.TmpNum]>0
        IF [NewNum.SubOrderBy]<>''
           CALC [NewNum.SubOrderBy] AS CON([NewNum.SubOrderBy],', ')
        END_IF

        CALC [NewNum.TmpStr] AS ''
        CALC [NewNum.LenVar] AS LEN([NewNum.WhereClause])
        WHILE [NewNum.TmpStr]<>' '
          INC [NewNum.TmpNum]
          IF [NewNum.TmpNum]>[NewNum.LenVar]
            CALC [NewNum.TmpStr] AS ' '
          ELSE
            CALC [NewNum.TmpStr] AS COPY([NewNum.WhereClause],[NewNum.TmpNum],1)
          END_IF
          IF [NewNum.TmpStr]<>' '
            CALC [NewNum.SubOrderBy] AS CON([NewNum.SubOrderBy],[NewNum.TmpStr])
          END_IF
        END_WHILE


        ; ! aus der Bedingung entfernen
        CALC [NewNum.TmpNum] AS POS('!',[NewNum.WhereClause])
        DEC  [NewNum.TmpNum]
        IF [NewNum.TmpNum]>0
          CALC [NewNum.TmpStr] AS COPY([NewNum.WhereClause],1,[NewNum.TmpNum])
        ELSE
          CALC [NewNum.TmpStr] AS ''
        END_IF
        CALC [NewNum.TmpNum] AS [NewNum.TmpNum] + 2
        CALC [NewNum.WhereClause] AS COPY([NewNum.WhereClause],[NewNum.TmpNum],255)
        IF [NewNum.WhereClause]=''
          CALC [NewNum.WhereClause] AS [NewNum.TmpStr]
          CALC [NewNum.TmpStr]    AS ''
        END_IF
        IF [NewNum.TmpStr]<>''
          CALC [NewNum.WhereClause] AS CON([NewNum.TmpStr],[NewNum.WhereClause])
        END_IF

        CALC [NewNum.TmpNum] AS POS('!',[NewNum.WhereClause])
      END_WHILE
    END_IF


    CALC [NewNum.StartValue]    AS [NewNum.AQuery!StartWert]
    CALC [NewNum.StepSize]      AS [NewNum.AQuery!Schrittweite]
    CALC [NewNum.Chars]         AS [NewNum.AQuery!Stellen]
    CALC [NewNum.FastMode]      AS [NewNum.AQuery!SchnellModus]

    CALC [NewNum.DataField]     AS TRIM([NewNum.AQuery!DatenFeld])
    CALC [NewNum.DataField]     AS PART([NewNum.DataField],1,' ')

    CALC [NewNum.DataTable]     AS [NewNum.AQuery!Datenbank]
    CALC [NewNum.VariabelStart] AS UPCASE([NewNum.AQuery!VariabelStart])

    CALC [NewNum.LenFix]        AS LEN([NewNum.AQuery!FestStart])
    CALC [NewNum.DefFld]        AS [NewNum.AQuery!FestStart]
    CALC [NewNum.DefFix]        AS [NewNum.AQuery!FestStart]

    IF [NewNum.VariabelStart]='JAHR'
      CALC [NewNum.DefVar] AS [DATE]
      CALC [NewNum.DefVar] AS COPY([NewNum.DefVar],7,4)
    ELSE_IF [NewNum.VariabelStart]='JAHR + MONAT'
      CALC [NewNum.DefVar] AS [DATE]
      CALC [NewNum.TmpStr] AS COPY([NewNum.DefVar],4,2)
      CALC [NewNum.DefVar] AS COPY([NewNum.DefVar],7,4)
      CALC [NewNum.DefVar] AS CON([NewNum.DefVar],[NewNum.TmpStr])
    ELSE_IF [NewNum.VariabelStart]='JAHR + MONAT + TAG'
      CALC [NewNum.SwpStr] AS [DATE]
      CALC [NewNum.DefVar] AS COPY([NewNum.SwpStr],7,4)
      CALC [NewNum.TmpStr] AS COPY([NewNum.SwpStr],4,2)
      CALC [NewNum.DefVar] AS CON([NewNum.DefVar],[NewNum.TmpStr])
      CALC [NewNum.TmpStr] AS COPY([NewNum.SwpStr],1,2)
      CALC [NewNum.DefVar] AS CON([NewNum.DefVar],[NewNum.TmpStr])
    ELSE_IF [NewNum.VariabelStart]='MONAT + TAG'
      CALC [NewNum.SwpStr] AS [DATE]
      CALC [NewNum.TmpStr] AS COPY([NewNum.SwpStr],4,2)
      CALC [NewNum.DefVar] AS CON([NewNum.DefVar],[NewNum.TmpStr])
      CALC [NewNum.TmpStr] AS COPY([NewNum.SwpStr],1,2)
      CALC [NewNum.DefVar] AS CON([NewNum.DefVar],[NewNum.TmpStr])
    ELSE_IF [NewNum.VariabelStart]='BENUTZER'
      CALC [NewNum.DefVar] AS COPY([SHORT_NAME],1,2)
    ELSE_IF [NewNum.VariabelStart]='JAHR 2 St.'
      CALC [NewNum.DefVar] AS [DATE]
      CALC [NewNum.DefVar] AS COPY([NewNum.DefVar],9,2)
    ELSE_IF [NewNum.VariabelStart]='JAHR 2 St. + MONAT'
      CALC [NewNum.DefVar] AS [DATE]
      CALC [NewNum.TmpStr] AS COPY([NewNum.DefVar],4,2)
      CALC [NewNum.DefVar] AS COPY([NewNum.DefVar],9,2)
      CALC [NewNum.DefVar] AS CON([NewNum.DefVar],[NewNum.TmpStr])
    ELSE_IF [NewNum.VariabelStart]='JAHR 2 St. + MONAT + TAG'
      CALC [NewNum.SwpStr] AS [DATE]
      CALC [NewNum.DefVar] AS COPY([NewNum.SwpStr],9,2)
      CALC [NewNum.TmpStr] AS COPY([NewNum.SwpStr],4,2)
      CALC [NewNum.DefVar] AS CON([NewNum.DefVar],[NewNum.TmpStr])
      CALC [NewNum.TmpStr] AS COPY([NewNum.SwpStr],1,2)
      CALC [NewNum.DefVar] AS CON([NewNum.DefVar],[NewNum.TmpStr])
    ELSE
      CALC [NewNum.DefVar] AS ''
    END_IF

    CALC [NewNum.LenVar]   AS LEN([NewNum.DefVar])
    CALC [NewNum.LenStart] AS [NewNum.LenVar]+[NewNum.LenFix]
    CALC [NewNum.DefFld]   AS CON([NewNum.DefFld],[NewNum.DefVar])
    CALC [NewNum.LenRest]  AS [NewNum.Chars]-[NewNum.LenStart]

    ; Start-Nummer erstellen
    CALC [NewNum.StartFld] AS EXT('0',[NewNum.LenRest],48,2,1)

    ; Ist ein Startwert in der Nummernkreisdefinition eingetragen
    IF [NewNum.StartValue]<>''
      CALC [NewNum.Result] AS FALSE

      IF [NewNum.StepSize]>0
        IF [NewNum.StartFld]<[NewNum.StartValue]
          CALC [NewNum.Result] AS TRUE
        END_IF
      ELSE
        IF [NewNum.StartFld]>[NewNum.StartValue]
          CALC [NewNum.Result] AS TRUE
        END_IF
      END_IF

      IF [NewNum.Result]
        CALC [NewNum.TmpNum] AS [NewNum.LenStart]+1
        CALC [NewNum.TmpStr] AS COPY([NewNum.StartValue],[NewNum.TmpNum],250)
        IF [NewNum.TmpStr]<>''
          CALC [NewNum.StartFld] AS EXT([NewNum.TmpStr],[NewNum.LenRest],48,2,1)
        END_IF
      END_IF
    END_IF

    ; Default Ergebnis ist Start-Nummer
    CALC [NewNum.ANumber] AS CON([NewNum.DefFld],[NewNum.StartFld])

    IF [NewNum.FastMode]=FALSE
      IF [NewNum.SubOrderBy]<>''
        CALC [NewNum.SubOrderBy] AS CON('ORDER BY ',[NewNum.SubOrderBy])
      END_IF

      BACK_MARK

      CALC [NewNum.TmpStr] AS ''
      IF [NewNum.DefFld]<>''
        IF [NewNum.WhereClause]<>''
          CALC [NewNum.TmpStr] AS ' AND '
        ELSE
          CALC [NewNum.TmpStr] AS ' WHERE '
        END_IF
        CALC [NewNum.TmpStr] AS CON([NewNum.TmpStr],' (',[NewNum.DataField],' LIKE "',[NewNum.DefFld],[SQL_WILD_CARD_MULTI],'")')
      END_IF

      IF [NewNum.StepSize]>0
        QUERY [NewNum.AQuery]
          SELECT MAX([NewNum.DataField])
        END_QUERY
      ELSE
        QUERY [NewNum.AQuery]
          SELECT MIN([NewNum.DataField])
        END_QUERY
      END_IF

      CALC [NewNum.SwpStr] AS CON('FROM ',[NewNum.DataTable])
      QUERY_ADD_STRING [NewNum.AQuery],[NewNum.SwpStr]
      QUERY_ADD_STRING [NewNum.AQuery],[NewNum.WhereClause]
      QUERY_ADD_STRING [NewNum.AQuery],[NewNum.TmpStr]
      QUERY_ADD_STRING [NewNum.AQuery],[NewNum.SubOrderBy]

      EXECUTE [NewNum.AQuery],[NewNum.Result]
      IF [NewNum.Result]=FALSE
        CALC [NewNum.YNRes] AS [ID_YES]
        GET_YES_NO 'Achtung! Fehler!','Das Tabellenfeld, das zum Zhlen verwendet wird, konnte nicht gelesen werden.|Soll der Vorgang wiederholt werden?',[NewNum.YNRes]
        IF [NewNum.YNRes]=[ID_YES]
          BACK_JUMP
        ELSE
          FORE_JUMP
        END_IF
      END_IF

      GET_DATA [NewNum.AQuery],[NewNum.Result]
      IF [NewNum.Result]
        IF [NewNum.StepSize]>0
          CALC [NewNum.NumRead] AS [NewNum.AQuery!MAX([NewNum.DataField])]
        ELSE
          CALC [NewNum.NumRead] AS [NewNum.AQuery!MIN([NewNum.DataField])]
        END_IF
      END_IF
    ELSE
      CALC [NewNum.NumRead] AS [NewNum.ANumber]
    END_IF

    IF [NewNum.VariabelStart]<>'Benutzer'
      ; Ergebnis kleiner als die Vorgabe (bei variablem Start kann das jedes Jahr etc. vorkommen)
      IF [NewNum.StepSize]>0
        IF [NewNum.NumRead]<[NewNum.ANumber]
          CALC [NewNum.NumRead] AS [NewNum.ANumber]
        END_IF
      ELSE
        IF [NewNum.NumRead]>[NewNum.NumberResult]
          CALC [NewNum.NumRead] AS [NewNum.ANumber]
        END_IF
      END_IF
    ELSE
      CALC [NewNum.TmpStr] AS ''
      IF [NewNum.LenStart]>0
        CALC [NewNum.TmpStr] AS COPY([NewNum.NumRead],1,[NewNum.LenStart])
      END_IF

      IF [NewNum.TmpStr]<>[NewNum.DefFld]
        CALC [NewNum.NumRead] AS [NewNum.ANumber]
      END_IF
    END_IF

    ; jetzt endlich zhlen!
    CALC [NewNum.TmpNum]   AS [NewNum.LenStart]+1
    CALC [NewNum.TmpStr]   AS COPY([NewNum.NumRead],[NewNum.TmpNum],250)
    CALC [NewNum.NumValue] AS VALUE([NewNum.TmpStr])
    CALC [NewNum.NumValue] AS ([NewNum.NumValue])+([NewNum.StepSize])
    CALC [NewNum.TmpStr]   AS [NewNum.NumValue]
    IF [NewNum.TmpStr]<>''
      CALC [NewNum.TmpStr] AS EXT([NewNum.TmpStr],[NewNum.LenRest],48,2,1)
    ELSE
      CALC [NewNum.TmpStr] AS EXT('1',[NewNum.LenRest],48,2,1)
    END_IF

    CALC [NewNum.ANumber] AS CON([NewNum.DefFld],[NewNum.TmpStr])



    ;******************************************************************************************
    ; Wurde eine neue Nummer erzeugt??!!
    ;******************************************************************************************
    IF [NewNum.ANumber]=''
      FORE_JUMP
    END_IF

    CALC [[NewNum.VarName]] AS [NewNum.ANumber]



    ;******************************************************************************************
    ; Passt das Ergebnis zu der vorherigen Nummer?
    ;******************************************************************************************
    CALC [NewNum.Result] AS FALSE
    IF [NewNum.DefFix]=''
      IF [NewNum.StepSize]>0
        IF [NewNum.NumRead]>[NewNum.ANumber]
          CALC [NewNum.Result] AS TRUE
        END_IF
      ELSE
        IF [NewNum.NumRead]<[NewNum.ANumber]
          CALC [NewNum.Result] AS TRUE
        END_IF
      END_IF
    END_IF


    IF [NewNum.Result]
      MESSAGE 'Achtung schwerwiegender Fehler (3)','Der Nummernkreis ist defekt!',[MESSAGE_ERROR]

      QUERY [NewNum.AQuery]
        UPDATE nummern
           SET EventFlag = 1
         WHERE RecordID  = [NewNum.NumDefRecID|R0,0]
      END_QUERY

      EXECUTE [NewNum.AQuery],[NewNum.Result]
      IF [NewNum.Result]=FALSE
        MESSAGE 'Achtung schwerwiegender Fehler (4)','Der Nummernkreis konnte nicht als defekt markiert werden!',[MESSAGE_ERROR]
      END_IF

      FORE_JUMP
    END_IF



    ;******************************************************************************************
    ; Update auf die Tabelle, wenn eine Datensatz-ID mit bergeben wurde
    ;******************************************************************************************
    IF [NewNum.FastMode]
      QUERY [NewNum.AQuery]
        UPDATE nummern
           SET StartWert = "[NewNum.ANumber]"
         WHERE RecordID  = [NewNum.NumDefRecID|R0,0]
      END_QUERY

      EXECUTE [NewNum.AQuery],[NewNum.Result]
      IF [NewNum.Result]=FALSE
        MESSAGE 'Achtung Fehler','Die neu ermittelte Nummer konnte dem aktuellen Datensatz nicht zugeordnet werden.',[MESSAGE_ERROR]
        FORE_JUMP
      END_IF
    ELSE_IF [NewNum.RecID]>[EPSILON]
      IF [NewNum.DataTable]<>''
        IF [NewNum.DataField]<>''
          QUERY [NewNum.AQuery]
            UPDATE [NewNum.DataTable]
               SET [NewNum.DataField] = "[NewNum.ANumber]"
             WHERE RecordID           = [NewNum.RecID|R0,0]
          END_QUERY

          EXECUTE [NewNum.AQuery],[NewNum.Result]
          IF [NewNum.Result]=FALSE
            MESSAGE 'Achtung Fehler','Die neu ermittelte Nummer konnte dem aktuellen Datensatz nicht zugeordnet werden.',[MESSAGE_ERROR]
            FORE_JUMP
          END_IF
        END_IF
      END_IF
    END_IF



    ;******************************************************************************************
    ; Wenn die Routine bis hierher fehlerlos ausgefhrt wurde, ist das Ergebnis GUT
    ;******************************************************************************************
    CALC [NewNum.MainRes] AS TRUE



    ;******************************************************************************************
    ; Nummernkreis wieder freigeben
    ;******************************************************************************************
    FORE_MARK

    IF [NewNum.MainRes]
      IF [NewNum.RecID]>[EPSILON]
        RECORD_LOCK 'nummern',[NewNum.NumDefRecID],FALSE,[NewNum.Result],FALSE
        CALC [RECORD_LOCKING_IS_ACTIVE] AS [NewNum.ActLocState]
      END_IF
    ELSE
      RECORD_LOCK 'nummern',[NewNum.NumDefRecID],FALSE,[NewNum.Result],FALSE
      CALC [RECORD_LOCKING_IS_ACTIVE] AS [NewNum.ActLocState]
    END_IF

    END_WAIT


    ;******************************************************************************************
    ; Wenn es eine Maske ist, MASK_UPDATE machen, das feld unmodified setzten und, ggf. das Feld fr Eingaben blocken
    ;******************************************************************************************
    IF [NewNum.MaskName]=''
      EXIT_PROC
    END_IF

    MASK_UPDATE [[NewNum.MaskName]]

    FIELD_FLAG_GET [[NewNum.MaskName]],[NewNum.MaskField],[NewNum.TmpNum]
    FLAG_SET [NewNum.TmpNum],[FIELD_FLAG_MODIFIED],FALSE

    FLAG_STATE [NewNum.TmpNum],[FIELD_FLAG_PROTECTED],[NewNum.FlagState]
    IF [NewNum.FlagState]=FALSE
      IF [NewNum.BlockField]
        FLAG_SET [NewNum.TmpNum],[FIELD_FLAG_PROTECTED],TRUE
      END_IF
    END_IF

    FIELD_FLAG_SET [[NewNum.MaskName]],[NewNum.MaskField],[NewNum.TmpNum]
  END_PROC



  ; Freigabe bzw. Entlocken eines Nummernkreises. Wenn FreeNum.VarName <> '' ist, wird die benannte Variable initialisiert.
  PROCEDURE FreeNum VN FreeNum.NumberID, VS FreeNum.VarName, ~VB FreeNum.MainRes
    LOCAL_QUERY FreeNum.AQuery
    LOCAL_NUM   FreeNum.TmpNum
    LOCAL_STR   FreeNum.MaskName
    LOCAL_STR   FreeNum.MaskField
    LOCAL_BOOL  FreeNum.FlagState

    QUERY [FreeNum.AQuery]
      SELECT RecordID FROM nummern WHERE KreisID = [FreeNum.NumberID]
    END_QUERY

    EXECUTE [FreeNum.AQuery], [FreeNum.MainRes]
    IF [FreeNum.MainRes]
      GET_DATA [FreeNum.AQuery], [FreeNum.MainRes]
      IF [FreeNum.MainRes]
        RECORD_LOCK 'nummern',[FreeNum.AQuery!RecordID],FALSE,[FreeNum.MainRes],FALSE
        CALC [RECORD_LOCKING_IS_ACTIVE] AS [NewNum.ActLocState]
      END_IF
    END_IF

    IF [FreeNum.VarName]=''
      EXIT_PROC
    END_IF

    CALC [[FreeNum.VarName]] AS ''

    CALC [FreeNum.TmpNum] AS POS('!',[FreeNum.VarName])
    IF [FreeNum.TmpNum]>0
      CALC [FreeNum.MaskName]  AS PART([FreeNum.VarName],1,'!')
      CALC [FreeNum.MaskField] AS PART([FreeNum.VarName],2,'!')
    END_IF

    IF [FreeNum.MaskName]=''
      EXIT_PROC
    END_IF

    MASK_UPDATE [[FreeNum.MaskName]]

    FIELD_FLAG_GET [[FreeNum.MaskName]],[FreeNum.MaskField],[FreeNum.TmpNum]
    FLAG_SET [FreeNum.TmpNum],[FIELD_FLAG_MODIFIED], FALSE
    FLAG_SET [FreeNum.TmpNum],[FIELD_FLAG_PROTECTED],FALSE
    FIELD_FLAG_SET [[FreeNum.MaskName]],[FreeNum.MaskField],[FreeNum.TmpNum]
  END_PROC



  ; Gibt in [BlkNum.Result] zurck, ob die Eingabefelder des Nummernkreises mit der ID [BlkNum.NumberID] geblockt werden sollen oder nicht.
  PROCEDURE BlkNum VAR_NUM BlkNum.NumberID, ~VAR_BOOL BlkNum.Result
    LOCAL_QUERY BlkNum.AQuery
    LOCAL_BOOL  BlkNum.AQueryRes

    CALC [BlkNum.Result] AS FALSE

    QUERY [BlkNum.AQuery]
      SELECT EingabeBlocken
        FROM Nummern
       WHERE KreisID = [BlkNum.NumberID]
    END_QUERY
    EXECUTE [BlkNum.AQuery],[BlkNum.AQueryRes]
    IF [BlkNum.AQueryRes]=FALSE
      MESSAGE 'Achtung : schwerer Datenbankfehler','Die Nummernkreistabelle kann nicht gelesen werden'
      EXIT_PROC
    END_IF

    GET_DATA [BlkNum.AQuery],[BlkNum.AQueryRes]
    IF [BlkNum.AQueryRes]=FALSE
      EXIT_PROC
    END_IF

    CALC [BlkNum.Result] AS [BlkNum.AQuery!EingabeBlocken]
  END_PROC



  ; Ist eine Nummer in einem gegebenen Feld in einer Tabelle schon vergeben. Diese Routine bezieht sich NICHT auf die Nummernkrteise
  ; denn sie soll auch dann noch funktionieren, wenn eien User einen Nummernkreis gelscht hat.
  PROCEDURE ChkNum  VS ChkNum.VarName, VF ChkNum.RecID, VS ChkNum.TName, VS ChkNum.FName, VS ChkNum.WClause, ~VB ChkNum.Result
    LOCAL_QUERY ChkNum.AQuery
    LOCAL_BOOL  ChkNum.AQueryRes

    CALC [ChkNum.Result] AS TRUE

    QUERY [ChkNum.AQuery]
      SELECT COUNT(RecordID)
        FROM [ChkNum.TName]
       WHERE [ChkNum.FName]    = "[[ChkNum.VarName]]"
             [ChkNum.WClause]
         AND RecordID         <> [ChkNum.RecID|R0,0]
    END_QUERY

    EXECUTE [ChkNum.AQuery],[ChkNum.AQueryRes]
    IF [ChkNum.AQueryRes]=FALSE
      MESSAGE 'Achtung : schwerer Datenbankfehler','Die Nummernprfung einer Tabelle konnte nicht ausgefhrt werden'
      EXIT_PROC
    END_IF

    GET_DATA [ChkNum.AQuery],[ChkNum.AQueryRes]
    IF [ChkNum.AQueryRes]=FALSE
      EXIT_PROC
    END_IF

    IF [ChkNum.AQuery!COUNT(RecordID)]>0
      CALC [ChkNum.Result] AS FALSE
    END_IF
  END_PROC
